from .event import *
from .bot import Bot as Bot
from .log import logger as logger
from .bot import run_bots as run_bots
from .store import get_bot as get_bot
from .store import get_bots as get_bots
from .bot import on_startup as on_startup
from .bot import on_shutdown as on_shutdown



from .models import ApiResponse

class ActionFailed(Exception):
    def __init__(self, status_code: int, response: ApiResponse):
        self.status_code = status_code
        self.response = response

    def __repr__(self) -> str:
        return (
            f"<ActionFailed: {self.status_code}, retcode={self.response.retcode}, "
            f"message={self.response.message}, data={self.response.data}>"
        )

    def __str__(self):
        return self.__repr__()


import sys
import logging
from typing import TYPE_CHECKING

import loguru

if TYPE_CHECKING:
    from loguru import Logger, Record

logger: "Logger" = loguru.logger


class LoguruHandler(logging.Handler):
    """logging 与 loguru 之间的桥梁，将 logging 的日志转发到 loguru。"""

    def emit(self, record: logging.LogRecord):
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        frame, depth = logging.currentframe(), 2
        while frame and frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level, record.getMessage()
        )


def default_filter(record: "Record"):
    """默认的日志过滤器，根据 `config.log_level` 配置改变日志等级。"""
    log_level = record["extra"].get("villa_log_level", "INFO")
    levelno = logger.level(log_level).no if isinstance(log_level, str) else log_level
    return record["level"].no >= levelno


default_format: str = (
    "<g>{time:MM-DD HH:mm:ss}</g> "
    "[<lvl>{level}</lvl>] "
    "<c><u>{name}</u></c> | "
    "{message}"
)
"""默认日志格式"""


def _log_patcher(record: "loguru.Record"):
    record["name"] = record["name"].split(".")[0]  # type: ignore


logger.remove()
logger_id = logger.add(
    sys.stdout,
    level=0,
    diagnose=False,
    filter=default_filter,
    format=default_format,
)



import re
import asyncio
from itertools import product
from typing import Any, Set, Dict, List, Type, Union, Literal, Optional

import httpx
import uvicorn
from fastapi import FastAPI
from pydantic import parse_obj_as
from fastapi.responses import JSONResponse

from .models import *
from .message import Message
from .exception import ActionFailed
from .message import MessageSegment
from .log import logger, _log_patcher
from .utils import run_sync, escape_tag
from .message import Link as LinkSegment
from .message import Post as PostSegment
from .message import Text as TextSegment
from .message import Image as ImageSegment
from .store import get_app, get_bot, store_bot
from .message import RoomLink as RoomLinkSegment
from .typing import T_Func, T_Handler, EventHandler
from .message import MentionAll as MentionAllSegment
from .message import MentionUser as MentionUserSegment
from .message import MentionRobot as MentionRobotSegment
from .event import Event, SendMessageEvent, event_classes, pre_handle_event


class Bot:
    """Villa Bot"""

    _event_handlers: List[EventHandler] = []
    _client: httpx.AsyncClient
    bot_id: str
    """机器人 Id"""
    bot_secret: str
    """机器人密钥"""
    callback_url: str
    """事件回调地址"""
    _bot_info: Optional[Robot] = None
    """机器人信息"""

    def __init__(self, bot_id: str, bot_secret: str, callback_url: str):
        """初始化一个 Bot 实例

        参数:
            bot_id: 机器人 ID
            bot_secret: 机器人密钥
            callback_url: 事件回调地址
        """
        self.bot_id: str = bot_id
        self.bot_secret: str = bot_secret
        self.callback_url: str = callback_url
        self._client = httpx.AsyncClient(
            base_url="https://bbs-api.miyoushe.com/vila/api/bot/platform/"
        )
        store_bot(self)

    @property
    def nickname(self) -> str:
        """Bot 昵称"""
        if self._bot_info is None:
            raise ValueError(f"Bot {self.bot_id} not connected")
        return self._bot_info.template.name

    @property
    def avatar_icon(self) -> str:
        """Bot 头像地址"""
        if self._bot_info is None:
            raise ValueError(f"Bot {self.bot_id} not connected")
        return self._bot_info.template.icon

    @property
    def commands(self) -> Optional[List[Command]]:
        """Bot 预设命令列表"""
        if self._bot_info is None:
            raise ValueError(f"Bot {self.bot_id} not connected")
        return self._bot_info.template.commands

    @property
    def description(self) -> str:
        """Bot 介绍"""
        if self._bot_info is None:
            raise ValueError(f"Bot {self.bot_id} not connected")
        return self._bot_info.template.desc

    @property
    def current_villa_id(self) -> int:
        """Bot 最后收到的事件的大别野 ID"""
        if self._bot_info is None:
            raise ValueError(f"Bot {self.bot_id} not connected")
        return self._bot_info.villa_id

    def on_event(
        self, *event_type: Type[Event], block: bool = False, priority: int = 1
    ):
        """注册一个事件处理函数

        当事件属于 event_type 中的任意一个时，执行处理函数。

        参数:
            *event_type: 事件类型列表.
            block: 是否阻止更低优先级的处理函数执行. 默认为 False.
            priority: 优先级. 默认为 1.
        """

        def _decorator(func: T_Handler) -> T_Handler:
            self._event_handlers.append(
                EventHandler(
                    event_type=event_type, func=func, block=block, priority=priority
                )
            )
            self._event_handlers.sort(key=lambda x: x.priority)
            return func

        return _decorator

    def on_message(self, block: bool = False, priority: int = 1):
        """注册一个消息事件处理函数

        当事件属于 SendMessageEvent 消息事件时，执行处理函数。

        参数:
            block: 是否阻止更低优先级的处理函数执行. 默认为 False.
            priority: 优先级. 默认为 1.
        """

        def _decorator(func: T_Handler) -> T_Handler:
            self._event_handlers.append(
                EventHandler(
                    event_type=(SendMessageEvent,),
                    func=func,
                    block=block,
                    priority=priority,
                )
            )
            self._event_handlers.sort(key=lambda x: x.priority)
            return func

        return _decorator

    def on_startswith(
        self,
        *startswith: str,
        prefix: Union[str, Set[str]] = set(""),
        block: bool = False,
        priority: int = 1,
    ):
        """注册一个消息事件处理函数

        当事件属于 SendMessageEvent 消息事件且纯文本部分以指定字符串开头时，执行处理函数。

        参数:
            *startswith: 字符串列表.
            prefix: 字符串前缀. 可以是字符串或字符串集合. 默认为 "".
            block: 是否阻止更低优先级的处理函数执行. 默认为 False.
            priority: 优先级. 默认为 1.
        """
        if isinstance(prefix, str):
            prefix = {prefix}
        startswith = tuple(set(p + s for p, s in list(product(prefix, startswith))))

        def _decorator(func: T_Handler) -> T_Handler:
            self._event_handlers.append(
                EventHandler(
                    event_type=(SendMessageEvent,),
                    func=func,
                    block=block,
                    priority=priority,
                    startswith=startswith or None,
                )
            )
            self._event_handlers.sort(key=lambda x: x.priority)
            return func

        return _decorator

    def on_endswith(self, *endswith: str, block: bool = False, priority: int = 1):
        """注册一个消息事件处理函数

        当事件属于 SendMessageEvent 消息事件且纯文本部分以指定字符串结尾时，执行处理函数。

        参数:
            *endswith: 字符串列表.
            block: 是否阻止更低优先级的处理函数执行. 默认为 False.
            priority: 优先级. 默认为 1.
        """

        def _decorator(func: T_Handler) -> T_Handler:
            self._event_handlers.append(
                EventHandler(
                    event_type=(SendMessageEvent,),
                    func=func,
                    block=block,
                    priority=priority,
                    endswith=endswith or None,
                )
            )
            self._event_handlers.sort(key=lambda x: x.priority)
            return func

        return _decorator

    def on_keyword(self, *keywords: str, block: bool = False, priority: int = 1):
        """注册一个消息事件处理函数

        当事件属于 SendMessageEvent 消息事件且纯文本部分包含指定关键词时，执行处理函数。

        参数:
            *keywords: 关键词列表.
            block: 是否阻止更低优先级的处理函数执行. 默认为 False.
            priority: 优先级. 默认为 1.
        """

        def _decorator(func: T_Handler) -> T_Handler:
            self._event_handlers.append(
                EventHandler(
                    event_type=(SendMessageEvent,),
                    func=func,
                    block=block,
                    priority=priority,
                    keywords=keywords or None,
                )
            )
            self._event_handlers.sort(key=lambda x: x.priority)
            return func

        return _decorator

    def on_regex(
        self, pattern: Union[str, re.Pattern], block: bool = False, priority: int = 1
    ):
        """注册一个消息事件处理函数

        当事件属于 SendMessageEvent 消息事件且纯文本部分与正则表达式匹配时，执行处理函数。

        参数:
            pattern: 正则表达式.
            block: 是否阻止更低优先级的处理函数执行. 默认为 False.
            priority: 优先级. 默认为 1.
        """
        if isinstance(pattern, str):
            pattern = re.compile(pattern)

        def _decorator(func: T_Handler) -> T_Handler:
            self._event_handlers.append(
                EventHandler(
                    event_type=(SendMessageEvent,),
                    func=func,
                    block=block,
                    priority=priority,
                    regex=pattern,
                )
            )
            self._event_handlers.sort(key=lambda x: x.priority)
            return func

        return _decorator

    async def send(
        self, villa_id: int, room_id: int, message: Union[str, Message, MessageSegment]
    ) -> str:
        """发送消息

        参数:
            villa_id: 大别野 ID
            room_id: 房间 ID
            message: 消息内容

        返回:
            str: 消息 ID
        """
        if isinstance(message, str):
            message = MessageSegment.text(message)
        if isinstance(message, MessageSegment):
            message = Message(message)
        content_info = await self._parse_message_content(message)
        if isinstance(content_info.content, TextMessageContent):
            object_name = "MHY:Text"
        elif isinstance(content_info.content, ImageMessageContent):
            object_name = "MHY:Image"
        else:
            object_name = "MHY:Post"
        return await self.send_message(
            villa_id=villa_id,
            room_id=room_id,
            object_name=object_name,
            msg_content=content_info.json(by_alias=True, exclude_none=True),
        )

    async def check_member_bot_access_token(
        self, token: str, villa_id: Optional[int] = None
    ) -> CheckMemberBotAccessTokenReturn:
        """校验用户机器人访问凭证，并返回用户信息

        参数:
            token: 用户机器人访问凭证
            villa_id: 大别野 ID. 默认为 None.

        返回:
            CheckMemberBotAccessTokenReturn: 用户信息
        """
        return CheckMemberBotAccessTokenReturn.parse_obj(
            await self._request(
                "GET",
                "checkMemberBotAccessToken",
                villa_id,
                json={"token": token},
            )
        )

    async def get_villa(self, villa_id: int) -> Villa:
        """获取大别野信息

        参数:
            villa_id: 大别野 ID

        返回:
            Villa: 大别野信息
        """
        return Villa.parse_obj(
            (await self._request("GET", "getVilla", villa_id, json={}))["villa"]
        )

    async def get_member(self, villa_id: int, uid: int) -> Member:
        """获取用户信息

        参数:
            villa_id: 大别野
            uid: 用户 ID

        返回:
            Member: 用户详情
        """
        return Member.parse_obj(
            (
                await self._request(
                    "GET",
                    "getMember",
                    villa_id,
                    json={"uid": uid},
                )
            )["member"]
        )

    async def get_villa_members(
        self, villa_id: int, offset: int, size: int
    ) -> MemberListReturn:
        """获取大别野成员列表

        参数:
            villa_id: 大别野 ID
            offset: 偏移量
            size: 分页大小

        返回:
            MemberListReturn: 大别野成员列表信息
        """
        return MemberListReturn.parse_obj(
            await self._request(
                "GET",
                "getVillaMembers",
                villa_id,
                json={"offset": offset, "size": size},
            )
        )

    async def delete_villa_member(self, villa_id: int, uid: int) -> None:
        """踢出大别野用户

        参数:
            villa_id: 大别野 ID
            uid: 用户 ID
        """
        await self._request(
            "POST",
            "deleteVillaMember",
            villa_id,
            json={"uid": uid},
        )

    async def pin_message(
        self, villa_id: int, msg_uid: str, is_cancel: bool, room_id: int, send_at: int
    ) -> None:
        """置顶消息

        参数:
            villa_id: 大别野 ID
            msg_uid: 消息 ID
            is_cancel: 是否取消置顶
            room_id: 房间 ID
            send_at: 消息发送时间
        """
        await self._request(
            "POST",
            "pinMessage",
            villa_id,
            json={
                "msg_uid": msg_uid,
                "is_cancel": is_cancel,
                "room_id": room_id,
                "send_at": send_at,
            },
        )

    async def recall_message(
        self, villa_id: int, msg_uid: str, room_id: int, msg_time: int
    ) -> None:
        """撤回消息

        参数:
            villa_id: 大别野 ID
            msg_uid: 消息 ID
            room_id: 房间 ID
            msg_time: 消息发送时间
        """
        await self._request(
            "POST",
            "recallMessage",
            villa_id,
            json={"msg_uid": msg_uid, "msg_time": msg_time, "room_id": room_id},
        )

    async def send_message(
        self,
        villa_id: int,
        room_id: int,
        object_name: str,
        msg_content: Union[str, MessageContentInfo],
    ) -> str:
        """发送消息

        参数:
            villa_id: 大别野 ID
            room_id: 房间 ID
            object_name: 消息类型
            msg_content: 将 MsgContentInfo 结构体序列化后的字符串

        返回:
            str: 消息 ID
        """
        if isinstance(msg_content, MessageContentInfo):
            content = msg_content.json(by_alias=True, exclude_none=True)
        else:
            content = msg_content
        return (
            await self._request(
                "POST",
                "sendMessage",
                villa_id,
                json={
                    "room_id": room_id,
                    "object_name": object_name,
                    "msg_content": content,
                },
            )
        )["bot_msg_id"]

    async def create_group(self, villa_id: int, group_name: str) -> int:
        """创建分组

        参数:
            villa_id: 大别野 ID
            group_name: 分组名称

        返回:
            int: 分组 ID
        """
        return (
            await self._request(
                "POST",
                "createGroup",
                villa_id,
                json={
                    "group_name": group_name,
                },
            )
        )["group_id"]

    async def edit_group(self, villa_id: int, group_id: int, group_name: str) -> None:
        """编辑分组

        参数:
            villa_id: 大别野 ID
            group_id: 分组 ID
            group_name: 分组名称
        """
        await self._request(
            "POST",
            "editGroup",
            villa_id,
            json={"group_id": group_id, "group_name": group_name},
        )

    async def delete_group(self, villa_id: int, group_id: int) -> None:
        """删除分组

        参数:
            villa_id: 大别野 ID
            group_id: 分组 ID
        """
        await self._request(
            "POST",
            "deleteGroup",
            villa_id,
            json={"group_id": group_id},
        )

    async def get_group_list(self, villa_id: int) -> List[Group]:
        """获取分组列表

        参数:
            villa_id: 大别野 ID

        返回:
            List[Group]: 分组列表
        """
        return parse_obj_as(
            List[Group],
            (await self._request("GET", "getGroupList", villa_id, json={}))["list"],
        )

    async def sort_group_list(self, villa_id: int, group_ids: List[int]) -> None:
        """分组列表排序

        参数:
            villa_id: 大别野 ID
            group_ids: 分组 ID 排序
        """
        await self._request(
            "POST",
            "sortGroupList",
            villa_id,
            json={"villa_id": villa_id, "group_ids": group_ids},
        )

    async def edit_room(self, villa_id: int, room_id: int, room_name: str) -> None:
        """编辑房间

        参数:
            villa_id: 大别野 ID
            room_id: 房间 ID
            room_name: 房间名称
        """
        await self._request(
            "POST",
            "editRoom",
            villa_id,
            json={"room_id": room_id, "room_name": room_name},
        )

    async def delete_room(self, villa_id: int, room_id: int) -> None:
        """删除房间

        参数:
            villa_id: 大别野 ID
            room_id: 房间 ID
        """
        await self._request(
            "POST",
            "deleteRoom",
            villa_id,
            json={"room_id": room_id},
        )

    async def get_room(self, villa_id: int, room_id: int) -> Room:
        """获取房间信息

        参数:
            villa_id: 大别野 ID
            room_id: 房间 ID

        返回:
            Room: 房间详情
        """
        return Room.parse_obj(
            (
                await self._request(
                    "GET",
                    "getRoom",
                    villa_id,
                    json={"room_id": room_id},
                )
            )["room"]
        )

    async def get_villa_group_room_list(self, villa_id: int) -> GroupRoom:
        """获取房间列表信息

        参数:
            villa_id: 大别野 ID

        返回:
            GroupRoom: 房间列表
        """
        return GroupRoom.parse_obj(
            (
                await self._request(
                    "GET",
                    "getVillaGroupRoomList",
                    villa_id,
                    json={},
                )
            )["list"]
        )

    async def sort_room_list(self, villa_id: int, room_list: List[RoomSort]) -> None:
        """房间列表排序

        参数:
            villa_id: 大别野 ID
            room_list: 期望的排序列表
        """
        await self._request(
            "POST",
            "sortRoomList",
            villa_id,
            json={
                "villa_id": villa_id,
                "room_list": [room.dict() for room in room_list],
            },
        )

    async def operate_member_to_role(
        self, villa_id: int, role_id: int, uid: int, is_add: bool
    ) -> None:
        """向身份组操作用户

        参数:
            villa_id: 大别野 ID
            role_id: 身份组 ID
            uid: 用户 ID
            is_add: 是否添加用户
        """
        await self._request(
            "POST",
            "operateMemberToRole",
            villa_id,
            json={"role_id": role_id, "uid": uid, "is_add": is_add},
        )

    async def create_member_role(
        self, villa_id: int, name: str, color: Color, permissions: List[Permission]
    ) -> int:
        """创建身份组

        参数:
            villa_id: 大别野 ID
            name: 身份组名称
            color: 身份组颜色
            permissions: 权限列表

        返回:
            int: 身份组 ID
        """
        return (
            await self._request(
                "POST",
                "createMemberRole",
                villa_id,
                json={"name": name, "color": str(color), "permissions": permissions},
            )
        )["id"]

    async def edit_member_role(
        self,
        villa_id: int,
        role_id: int,
        name: str,
        color: Color,
        permissions: List[Permission],
    ) -> None:
        """编辑身份组

        参数:
            villa_id: 大别野 ID
            role_id: 身份组 ID
            name: 身份组名称
            color: 身份组颜色
            permissions: 权限列表
        """
        await self._request(
            "POST",
            "editMemberRole",
            villa_id,
            json={
                "id": role_id,
                "name": name,
                "color": str(color),
                "permissions": permissions,
            },
        )

    async def delete_member_role(self, villa_id: int, role_id: int) -> None:
        """删除身份组

        参数:
            villa_id: 大别野 ID
            role_id: 身份组 ID
        """
        await self._request(
            "POST",
            "deleteMemberRole",
            villa_id,
            json={"id": role_id},
        )

    async def get_member_role_info(
        self, villa_id: int, role_id: int
    ) -> MemberRoleDetail:
        """获取身份组

        参数:
            villa_id: 大别野 ID
            role_id: 身份组 ID

        返回:
            MemberRoleDetail: 身份组详情
        """
        return MemberRoleDetail.parse_obj(
            (
                await self._request(
                    "GET",
                    "getMemberRoleInfo",
                    villa_id,
                    json={"role_id": role_id},
                )
            )["role"]
        )

    async def get_villa_member_roles(self, villa_id: int) -> List[MemberRoleDetail]:
        """获取大别野下所有身份组

        参数:
            villa_id: 大别野 ID

        返回:
            List[MemberRoleDetail]: 身份组列表
        """
        return parse_obj_as(
            List[MemberRoleDetail],
            (
                await self._request(
                    "GET",
                    "getVillaMemberRoles",
                    villa_id,
                    json={},
                )
            )["list"],
        )

    async def get_all_emoticons(self) -> List[Emoticon]:
        """获取全量表情

        参数:
            villa_id: 参数说明

        返回:
            List[Emoticon]: 表情列表
        """
        return parse_obj_as(
            List[Emoticon],
            (
                await self._request(
                    "GET",
                    "getAllEmoticons",
                    None,
                    json={},
                )
            )["list"],
        )

    async def audit(
        self,
        villa_id: int,
        audit_content: str,
        pass_through: Optional[str] = None,
        room_id: Optional[int] = None,
        uid: Optional[int] = None,
    ) -> int:
        """审核

        审核用户配置内容是否合规，调用成功后会返回审核事件id(audit_id)。审核结果会通过回调接口异步通知。

        参数:
            villa_id: 大别野 ID
            audit_content: 待审核内容
            pass_through: 透传信息，该字段会在审核结果回调时携带给开发者，选填
            room_id: 房间 id，选填
            uid: 用户 id, 选填明

        返回:
            int: 审核事件 ID
        """
        return (
            await self._request(
                "POST",
                "audit",
                villa_id,
                json={
                    "audit_content": audit_content,
                    "pass_through": pass_through,
                    "room_id": room_id,
                    "uid": uid,
                },
            )
        )["audit_id"]

    def _get_headers(self, villa_id: Optional[int] = None) -> Dict[str, str]:
        """获取鉴权请求头

        参数:
            villa_id: 大别野 ID，部分无需

        返回:
            Dict[str, str]: 请求头
        """
        return {
            "x-rpc-bot_id": self.bot_id,
            "x-rpc-bot_secret": self.bot_secret,
            "x-rpc-bot_villa_id": str(villa_id) if villa_id else "",
        }

    async def _request(
        self,
        method: Literal["GET", "POST"],
        api: str,
        villa_id: Optional[int],
        json: Dict[str, Any],
        **kwargs,
    ) -> Any:
        """请求 API

        参数:
            method: 请求方法
            api: API 名称
            villa_id: 大别野 ID
            json: JSON请求体

        异常:
            ActionFailed: 动作失败
            e: 其他请求异常

        返回:
            Any: 返回结果
        """
        logger.opt(colors=True).debug(
            f"<b><m>{self.bot_id}</m></b> | Calling API <y>{api}</y>"
        )
        try:
            resp = await self._client.request(
                method=method,
                url=api,
                headers=self._get_headers(villa_id),
                json=json,
                **kwargs,
            )
            resp = ApiResponse.parse_raw(resp.content)
            if resp.retcode == 0:
                return resp.data
            else:
                raise ActionFailed(resp.retcode, resp)
        except Exception as e:
            raise e

    async def _close_client(self) -> None:
        """关闭 HTTP Client"""
        await self._client.aclose()

    async def _handle_event(self, event: Event):
        """处理事件

        参数:
            event: 事件
        """
        is_handled = False
        for event_handler in self._event_handlers:
            if isinstance(event, event_handler.event_type):
                if isinstance(event, SendMessageEvent):
                    if (
                        event_handler.startswith is not None
                        and not event.message.startswith(event_handler.startswith)
                    ):
                        logger.opt(colors=True).trace(
                            f"<b><y>[{event.__class__.__name__}]</y></b> not startswith \"{'|'.join(event_handler.startswith)}\", pass"
                        )
                        continue
                    if (
                        event_handler.endswith is not None
                        and not event.message.endswith(event_handler.endswith)
                    ):
                        logger.opt(colors=True).trace(
                            f"<b><y>[{event.__class__.__name__}]</y></b> not endswith \"{'|'.join(event_handler.endswith)}\", pass"
                        )
                        continue
                    if event_handler.keywords is not None and not any(
                        keyword in event.message for keyword in event_handler.keywords
                    ):
                        logger.opt(colors=True).trace(
                            f"<b><y>[{event.__class__.__name__}]</y></b> not contains \"{'|'.join(event_handler.keywords)}\", pass"
                        )
                        continue
                    if event_handler.regex is not None and not event.message.match(
                        event_handler.regex
                    ):
                        logger.opt(colors=True).trace(
                            f'<b><y>[{event.__class__.__name__}]</y></b> not match "{event_handler.regex}", <y>pass</y>'
                        )
                        continue
                logger.opt(colors=True).info(
                    f"<b><y>[{event.__class__.__name__}]</y></b> will be handled by <y>{event_handler}</y>"
                )
                await run_handler(event_handler, event)
                is_handled = True
                if event_handler.block:
                    logger.opt(colors=True).debug(
                        f"<b><y>[{event.__class__.__name__}]</y></b> stop handled by <y>{event_handler}</y>"
                    )
                    break
        if is_handled:
            logger.opt(colors=True).success(
                f"<b><y>[{event.__class__.__name__}]</y></b> handle completed"
            )

    async def _parse_message_content(self, message: Message) -> MessageContentInfo:
        """解析消息内容"""
        if quote := message["quote", 0]:
            quote = QuoteInfo(**quote.dict())

        if images_msg := (message["image"] or None):  # type: ignore
            images_msg: List[ImageSegment]
            images = [
                Image(
                    url=seg.url,
                    size=ImageSize(width=seg.width, height=seg.height)
                    if seg.width and seg.height
                    else None,
                    file_size=seg.file_size,
                )
                for seg in images_msg
            ]
        else:
            images = None
        if posts_msg := (message["post"] or None):  # type: ignore
            posts_msg: List[PostSegment]
            post_ids = [seg.post_id for seg in posts_msg]
        else:
            post_ids = None
        cal_len = lambda x: len(x.encode("utf-16")) // 2 - 1
        message_text = ""
        message_offset = 0
        entities: List[TextEntity] = []
        mentioned = MentionedInfo(type=MentionType.PART)
        for seg in message:  # type: ignore
            try:
                if seg.type in ("quote", "image", "post"):
                    continue
                if isinstance(seg, TextSegment):
                    seg_text = seg.content
                    length = cal_len(seg_text)
                elif isinstance(seg, MentionAllSegment):
                    seg_text = f"@{seg.show_text} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=MentionedAll(show_text=seg.show_text),
                        )
                    )
                    mentioned.type = MentionType.ALL
                elif isinstance(seg, MentionRobotSegment):
                    seg_text = f"@{seg.bot_name} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=MentionedRobot(
                                bot_id=seg.bot_id, bot_name=seg.bot_name
                            ),
                        )
                    )
                    mentioned.user_id_list.append(seg.bot_id)
                elif isinstance(seg, MentionUserSegment):
                    # 需要调用API获取被@的用户的昵称
                    user = await self.get_member(villa_id=seg.villa_id, uid=seg.user_id)
                    seg_text = f"@{user.basic.nickname} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=MentionedUser(
                                user_id=str(user.basic.uid),
                                user_name=user.basic.nickname,
                            ),
                        )
                    )
                    mentioned.user_id_list.append(str(user.basic.uid))
                elif isinstance(seg, RoomLinkSegment):
                    # 需要调用API获取房间的名称
                    room = await self.get_room(
                        villa_id=seg.villa_id, room_id=seg.room_id
                    )
                    seg_text = f"#{room.room_name} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=VillaRoomLink(
                                villa_id=str(seg.villa_id),
                                room_id=str(seg.room_id),
                                room_name=room.room_name,
                            ),
                        )
                    )
                else:
                    seg: LinkSegment
                    seg_text = seg.show_text
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=Link(url=seg.url, show_text=seg.show_text),
                        )
                    )
                message_offset += length
                message_text += seg_text
            except Exception as e:
                logger.opt(exception=e).warning("error when parse message content")

        if not (mentioned.type == MentionType.ALL and mentioned.user_id_list):
            mentioned = None

        if not (message_text or entities):
            if images:
                if len(images) > 1:
                    logger.warning(
                        "Sending multiple images in one message will not be visible on the web side!"
                    )
                    content = TextMessageContent(text="\u200B", images=images)
                else:
                    content = ImageMessageContent(**images[0].dict())
            elif post_ids:
                if len(post_ids) > 1:
                    logger.opt(colors=True).warning(
                        f"Only support one post in one message, so use the last one <m>{post_ids[-1]}</m>!"
                    )
                content = PostMessageContent(post_id=post_ids[-1])
            else:
                raise ValueError("message content is empty")
        else:
            if images and message_text:
                logger.warning(
                    "When a message is accompanied by text and image, image will not visible on the web side!"
                )
            if post_ids and message_text:
                logger.warning(
                    "When a message is accompanied by text and post, post will not visible!"
                )
            content = TextMessageContent(
                text=message_text, entities=entities, images=images
            )

        return MessageContentInfo(
            content=content,
            mentionedInfo=mentioned,
            quote=quote,
        )

    def init_app(self, app: FastAPI):
        logger.opt(colors=True).info(f"Initializing Bot <m>{self.bot_id}</m>...")
        logger.opt(colors=True).debug(
            f"With Secret: <m>{self.bot_secret}</m> and Callback URL: <m>{self.callback_url}</m>"
        )
        app.post(self.callback_url, status_code=200)(handle_event)
        app.on_event("shutdown")(self._close_client)

    def run(
        self,
        host: str = "127.0.0.1",
        port: int = 13350,
        log_level: str = "INFO",
        **kwargs,
    ):
        """启动机器人.

        参数:
            host: HOST 地址. 默认为 "127.0.0.1".
            port: 端口号. 默认为 13350.
            log_level: 日志等级. 默认为 "INFO".
        """
        run_bots(bots=[self], host=host, port=port, log_level=log_level, **kwargs)


def run_bots(
    bots: List[Bot],
    host: str = "127.0.0.1",
    port: int = 13350,
    log_level: str = "INFO",
    **kwargs,
):
    """启动多个机器人.

    参数:
        bots: 机器人列表.
        host: HOST 地址. 默认为 "127.0.0.1".
        port: 端口号. 默认为 13350.
        log_level: 日志等级. 默认为 "INFO".
    """
    logger.configure(extra={"villa_log_level": log_level}, patcher=_log_patcher)
    logger.success("Starting Villa...")
    fastapi_kwargs = {
        k.lstrip("fastapi_"): v for k, v in kwargs.items() if k.startswith("fastapi_")
    }
    uvicorn_kwargs = {
        k.lstrip("uvicorn_"): v for k, v in kwargs.items() if k.startswith("uvicorn_")
    }
    app = get_app()
    for key, value in fastapi_kwargs.items():
        setattr(app, key, value)
    for bot in bots:
        bot.init_app(app)
    uvicorn.run(
        app,
        host=host,
        port=port,
        log_config={
            "version": 1,
            "disable_existing_loggers": False,
            "handlers": {
                "default": {
                    "class": "villa.log.LoguruHandler",
                },
            },
            "loggers": {
                "uvicorn.error": {"handlers": ["default"], "level": "INFO"},
                "uvicorn.access": {
                    "handlers": ["default"],
                    "level": "INFO",
                },
            },
        },
        **uvicorn_kwargs,
    )


async def handle_event(data: Dict[str, Any]) -> JSONResponse:
    """处理事件"""
    if not (payload_data := data.get("event", None)):
        logger.warning(f"Received invalid data: {escape_tag(str(data))}")
        return JSONResponse(
            status_code=400, content={"retcode": -1, "msg": "Invalid data"}
        )
    try:
        event = parse_obj_as(event_classes, pre_handle_event(payload_data))
        if (bot := get_bot(event.bot_id)) is None:
            raise ValueError(f"Bot {event.bot_id} not found")
        bot._bot_info = event.robot
        logger.opt(colors=True).success(
            f"<b><m>{bot.bot_id}</m></b> | <b><y>[{event.__class__.__name__}]</y></b>: {event.get_event_description()}"
        )
    except Exception as e:
        logger.opt(exception=e).warning(
            f"Failed to parse payload {escape_tag(str(payload_data))}"
        )
        return JSONResponse(
            status_code=400, content={"retcode": -1, "msg": "Invalid data"}
        )
    else:
        asyncio.create_task(bot._handle_event(event))
    return JSONResponse(status_code=200, content={"retcode": 0, "message": "success"})


async def run_handler(handler: EventHandler, event: Event):
    """运行事件处理器"""
    try:
        if asyncio.iscoroutinefunction(handler.func):
            await handler.func(event)
        else:
            await run_sync(handler.func)(event)
    except Exception as e:
        logger.opt(exception=e).error(
            f"Error when running {handler} for {event.__class__.__name__}"
        )


def on_startup(func: T_Func):
    """让函数在 APP 启动时运行

    参数:
        func: 无参函数
    """
    get_app().on_event("startup")(func)


def on_shutdown(func: T_Func):
    """让函数在 APP 终止前运行

    参数:
        func: 无参函数
    """
    get_app().on_event("shutdown")(func)



import json
from enum import IntEnum
from typing import Any, Dict, Union, Literal, Optional

from pydantic import BaseModel, root_validator

from .store import get_bot
from .utils import escape_tag
from .message import Message, MessageSegment
from .models import Robot, MessageContentInfo


class EventType(IntEnum):
    """事件类型"""

    JoinVilla = 1
    SendMessage = 2
    CreateRobot = 3
    DeleteRobot = 4
    AddQuickEmoticon = 5
    AuditCallback = 6


class AuditResult(IntEnum):
    """审核结果类型"""

    Compatibility = 0
    """兼容"""
    Pass = 1
    """通过"""
    Reject = 2
    """驳回"""


class Event(BaseModel):
    """Villa 事件基类"""

    robot: Robot
    """用户机器人访问凭证"""
    type: EventType
    """事件类型"""
    id: str
    """事件 id"""
    created_at: int
    """事件创建时间"""
    send_at: int
    """事件回调时间"""

    @property
    def bot_id(self) -> str:
        """机器人ID"""
        return self.robot.template.id

    def get_event_description(self) -> str:
        return escape_tag(str(self.dict()))


class JoinVillaEvent(Event):
    """新用户加入大别野事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###JoinVilla"""

    type: Literal[EventType.JoinVilla] = EventType.JoinVilla
    join_uid: int
    """用户ID"""
    join_user_nickname: str
    """用户昵称"""
    join_at: int
    """用户加入时间的时间戳"""

    @property
    def villa_id(self) -> int:
        """大别野ID"""
        return self.robot.villa_id

    def get_event_description(self) -> str:
        return escape_tag(
            f"User(nickname={self.join_user_nickname},id={self.join_uid}) join Villa(id={self.villa_id})"
        )


class SendMessageEvent(Event):
    """用户@机器人发送消息事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###SendMessage"""

    type: Literal[EventType.SendMessage] = EventType.SendMessage
    content: MessageContentInfo
    """消息内容"""
    from_user_id: int
    """发送者ID"""
    send_at: int
    """发送时间的时间戳"""
    room_id: int
    """房间ID"""
    object_name: int
    """目前只支持文本类型消息"""
    nickname: str
    """用户昵称"""
    msg_uid: str
    """消息ID"""
    bot_msg_id: Optional[str]
    """如果被回复的消息从属于机器人，则该字段不为空字符串"""

    message: Message
    """事件消息"""

    @property
    def villa_id(self) -> int:
        """大别野ID"""
        return self.robot.villa_id

    def get_event_description(self) -> str:
        return escape_tag(
            f"Message(id={self.msg_uid}) was sent from User(nickname={self.nickname},id={self.from_user_id}) in Room(id={self.room_id}) of Villa(id={self.villa_id}), content={repr(self.message)}"
        )

    @root_validator(pre=True)
    def _(cls, data: Dict[str, Any]):
        if not data.get("content"):
            return data
        msg = Message()
        data["content"] = json.loads(data["content"])
        msg_content_info = data["content"]
        if quote := msg_content_info.get("quote"):
            msg.append(
                MessageSegment.quote(
                    message_id=quote["quoted_message_id"],
                    message_send_time=quote["quoted_message_send_time"],
                )
            )

        content = msg_content_info["content"]
        text = content["text"]
        entities = content["entities"]
        if not entities:
            return Message(MessageSegment.text(text))
        text = text.encode("utf-16")
        last_offset: int = 0
        last_length: int = 0
        for entity in entities:
            end_offset: int = last_offset + last_length
            offset: int = entity["offset"]
            length: int = entity["length"]
            entity_detail = entity["entity"]
            if offset != end_offset:
                msg.append(
                    MessageSegment.text(
                        text[((end_offset + 1) * 2) : ((offset + 1) * 2)].decode(
                            "utf-16"
                        )
                    )
                )
            entity_text = text[(offset + 1) * 2 : (offset + length + 1) * 2].decode(
                "utf-16"
            )
            if entity_detail["type"] == "mentioned_robot":
                entity_detail["bot_name"] = entity_text.lstrip("@")[:-1]
                msg.append(
                    MessageSegment.mention_robot(
                        entity_detail["bot_id"], entity_detail["bot_name"]
                    )
                )
            elif entity_detail["type"] == "mentioned_user":
                entity_detail["user_name"] = entity_text.lstrip("@")[:-1]
                msg.append(
                    MessageSegment.mention_user(
                        int(entity_detail["user_id"]), data["villa_id"]
                    )
                )
            elif entity_detail["type"] == "mention_all":
                entity_detail["show_text"] = entity_text.lstrip("@")[:-1]
                msg.append(MessageSegment.mention_all(entity_detail["show_text"]))
            elif entity_detail["type"] == "villa_room_link":
                entity_detail["room_name"] = entity_text.lstrip("#")[:-1]
                msg.append(
                    MessageSegment.room_link(
                        int(entity_detail["villa_id"]),
                        int(entity_detail["room_id"]),
                    )
                )
            else:
                entity_detail["show_text"] = entity_text
                msg.append(MessageSegment.link(entity_detail["url"], entity_text))
            last_offset = offset
            last_length = length
        end_offset = last_offset + last_length
        if last_text := text[(end_offset + 1) * 2 :].decode("utf-16"):
            msg.append(MessageSegment.text(last_text))
        data["message"] = msg
        return data

    async def send(
        self,
        message: Union[str, MessageSegment, Message],
        mention_sender: bool = False,
        quote_message: bool = False,
    ) -> str:
        """对事件进行快速发送消息

        参数:
            message: 消息内容
            mention_sender: 是否@发送者. 默认为 False.
            quote_message: 是否引用事件消息. 默认为 False.

        异常:
            ValueError: 找不到 Bot 实例

        返回:
            str: 消息 ID
        """
        if (bot := get_bot(self.bot_id)) is None:
            raise ValueError(f"Bot {self.bot_id} not found. Cannot send message.")
        if isinstance(message, (str, MessageSegment)):
            message = Message(message)
        if mention_sender:
            message.insert(
                0, MessageSegment.mention_user(self.from_user_id, self.villa_id)
            )
        if quote_message:
            message.append(MessageSegment.quote(self.msg_uid, self.send_at))
        return await bot.send(self.villa_id, self.room_id, message)


class CreateRobotEvent(Event):
    """大别野添加机器人实例事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###CreateRobot"""

    type: Literal[EventType.CreateRobot] = EventType.CreateRobot
    villa_id: int
    """大别野ID"""

    def get_event_description(self) -> str:
        return escape_tag(
            f"Bot(id={self.bot_id}) was added to Villa(id={self.villa_id})"
        )


class DeleteRobotEvent(Event):
    """大别野删除机器人实例事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###DeleteRobot"""

    type: Literal[EventType.DeleteRobot] = EventType.DeleteRobot
    villa_id: int
    """大别野ID"""

    def get_event_description(self) -> str:
        return escape_tag(
            f"Bot(id={self.bot_id}) was removed from Villa(id={self.villa_id})"
        )


class AddQuickEmoticonEvent(Event):
    """用户使用表情回复消息表态事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html#AddQuickEmoticon"""

    type: Literal[EventType.AddQuickEmoticon] = EventType.AddQuickEmoticon
    villa_id: int
    """大别野ID"""
    room_id: int
    """房间ID"""
    uid: int
    """发送表情的用户ID"""
    emoticon_id: int
    """表情ID"""
    emoticon: str
    """表情内容"""
    msg_uid: str
    """被回复的消息 id"""
    bot_msg_id: Optional[str]
    """如果被回复的消息从属于机器人，则该字段不为空字符串"""
    is_cancel: bool = False
    """是否是取消表情"""

    def get_event_description(self) -> str:
        return escape_tag(
            f"Emoticon(name={self.emoticon}, id={self.emoticon_id}) was {'removed from' if self.is_cancel else 'added to'} Message(id={self.msg_uid}) by User(id={self.uid}) in Room(id=Villa(id={self.room_id}) of Villa(id={self.villa_id})"
        )


class AuditCallbackEvent(Event):
    """审核结果回调事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html#AuditCallback"""

    type: Literal[EventType.AuditCallback] = EventType.AuditCallback
    audit_id: str
    """审核事件 id"""
    bot_tpl_id: str
    """机器人 id"""
    villa_id: int
    """大别野 ID"""
    room_id: int
    """房间 id（和审核接口调用方传入的值一致）"""
    user_id: int
    """用户 id（和审核接口调用方传入的值一致）"""
    pass_through: str
    """透传数据（和审核接口调用方传入的值一致）"""
    audit_result: AuditResult
    """审核结果"""

    def get_event_description(self) -> str:
        return escape_tag(
            f"Audit(id={self.audit_id},result={self.audit_result}) of User(id={self.user_id}) in Room(id={self.room_id}) of Villa(id={self.villa_id})"
        )


event_classes = Union[
    JoinVillaEvent,
    SendMessageEvent,
    CreateRobotEvent,
    DeleteRobotEvent,
    AddQuickEmoticonEvent,
    AuditCallbackEvent,
]


def pre_handle_event(payload: Dict[str, Any]):
    if (event_type := EventType._value2member_map_.get(payload["type"])) is None:
        raise ValueError(
            f"Unknown event type: {payload['type']} data={escape_tag(str(payload))}"
        )
    event_name = event_type.name
    if event_name not in payload["extend_data"]["EventData"]:
        raise ValueError("Cannot find event data for event type: {event_name}")
    payload |= payload["extend_data"]["EventData"][event_name]
    payload.pop("extend_data")
    return payload


__all__ = [
    "Event",
    "JoinVillaEvent",
    "SendMessageEvent",
    "CreateRobotEvent",
    "DeleteRobotEvent",
    "AddQuickEmoticonEvent",
    "AuditCallbackEvent",
]



import re
from typing import Any, Type, Tuple, Union, TypeVar, Callable, Optional, Awaitable

from pydantic import BaseModel

from .event import Event

T_Event = TypeVar("T_Event", bound=Event)


T_Handler = Union[Callable[[T_Event], Any], Callable[[T_Event], Awaitable[Any]]]
T_Func = Callable[..., Any]


class EventHandler(BaseModel):
    event_type: Tuple[Type[Event], ...]
    func: T_Handler
    priority: int = 1
    block: bool = False

    startswith: Optional[Tuple[str, ...]] = None
    endswith: Optional[Tuple[str, ...]] = None
    keywords: Optional[Tuple[str, ...]] = None
    regex: Optional[re.Pattern] = None

    def __str__(self) -> str:
        return f"Handler(func={self.func.__name__}, priority={self.priority}, block={self.block})"

    def __repr__(self) -> str:
        return super().__str__()



from typing import TYPE_CHECKING, Dict, Optional

from fastapi import FastAPI

if TYPE_CHECKING:
    from .bot import Bot

_bots: Dict[str, "Bot"] = {}
_app: FastAPI = FastAPI()


def get_bot(bot_id: Optional[str] = None) -> Optional["Bot"]:
    """获取一个 Bot 实例

    如果没有 Bot，则返回 None。
    如果没有指定 bot_id，则返回第一个 Bot 实例。

    返回:
        _type_: Bot
    """
    if not _bots:
        return None
    if bot_id is None:
        return _bots[list(_bots.keys())[0]]
    return _bots.get(bot_id)


def get_bots() -> Dict[str, "Bot"]:
    """获取所有 Bot 实例"""
    return _bots


def store_bot(bot: "Bot") -> None:
    if bot.bot_id in _bots:
        raise ValueError(f"Bot {bot.bot_id} already in bots")
    _bots[bot.bot_id] = bot


def get_app() -> FastAPI:
    """获取 FastAPI 实例"""
    return _app


__all__ = ["get_bot", "get_bots", "store_bot", "get_app"]



import re
import asyncio
from functools import partial, wraps
from typing import Callable, TypeVar, Coroutine
from typing_extensions import ParamSpec

P = ParamSpec("P")
R = TypeVar("R")

def escape_tag(s: str) -> str:
    """用于记录带颜色日志时转义 `<tag>` 类型特殊标签

    参考: [loguru color 标签](https://loguru.readthedocs.io/en/stable/api/logger.html#color)

    参数:
        s: 需要转义的字符串
    """
    return re.sub(r"</?((?:[fb]g\s)?[^<>\s]*)>", r"\\\g<0>", s)


def run_sync(call: Callable[P, R]) -> Callable[P, Coroutine[None, None, R]]:
    """一个用于包装 sync function 为 async function 的装饰器

    参数:
        call: 被装饰的同步函数
    """

    @wraps(call)
    async def _wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        loop = asyncio.get_running_loop()
        pfunc = partial(call, *args, **kwargs)
        result = await loop.run_in_executor(None, pfunc)
        return result

    return _wrapper
    


import re
from abc import ABC
from typing_extensions import Self
from typing import List, Tuple, Union, Literal, Iterable, Iterator, Optional, overload

from pydantic import Field, BaseModel

MessageType = Literal[
    "text",
    "mention_user",
    "mention_all",
    "mention_robot",
    "room_link",
    "link",
    "image",
    "quote",
    "post",
]


class MessageSegment(ABC, BaseModel):
    type: MessageType
    """消息段基类"""

    @staticmethod
    def text(text: str) -> "Text":
        return Text(content=text)

    @staticmethod
    def mention_robot(bot_id: str, bot_name: str) -> "MentionRobot":
        return MentionRobot(bot_id=bot_id, bot_name=bot_name)

    @staticmethod
    def mention_user(villa_id: int, user_id: int) -> "MentionUser":
        return MentionUser(villa_id=villa_id, user_id=user_id)

    @staticmethod
    def mention_all(show_text: str = "全体成员") -> "MentionAll":
        return MentionAll(show_text=show_text)

    @staticmethod
    def room_link(villa_id: int, room_id: int) -> "RoomLink":
        return RoomLink(villa_id=villa_id, room_id=room_id)

    @staticmethod
    def link(url: str, show_text: Optional[str] = None) -> "Link":
        return Link(url=url, show_text=show_text or url)

    @staticmethod
    def post(post_id: str) -> "Post":
        return Post(post_id=post_id)

    @staticmethod
    def image(
        url: str,
        width: Optional[int] = None,
        height: Optional[int] = None,
        file_size: Optional[int] = None,
    ) -> "Image":
        return Image(url=url, width=width, height=height, file_size=file_size)

    @staticmethod
    def quote(message_id: str, message_send_time: int) -> "Quote":
        return Quote(
            quoted_message_id=message_id,
            quoted_message_send_time=message_send_time,
            original_message_id=message_id,
            original_message_send_time=message_send_time,
        )

    def __add__(self, other: Union[str, "MessageSegment", "Message"]) -> "Message":
        if isinstance(other, str):
            return Message([self, MessageSegment.text(other)])
        elif isinstance(other, MessageSegment):
            return Message([self, other])
        elif isinstance(other, Message):
            return Message([self, *other.__root__])
        else:
            raise TypeError(f"unsupported type: {type(other)}")

    def __radd__(self, other: Union[str, "MessageSegment", "Message"]) -> "Message":
        return self.__add__(other)

    def __iadd__(self, other: Union[str, "MessageSegment", "Message"]) -> "Message":
        return self.__add__(other)


class Text(MessageSegment):
    """文本消息段"""

    type: Literal["text"] = Field(default="text", repr=False)
    content: str


class MentionRobot(MessageSegment):
    """@机器人消息段"""

    type: Literal["mention_robot"] = Field(default="mention_robot", repr=False)
    bot_id: str
    bot_name: str


class MentionUser(MessageSegment):
    """@用户消息段"""

    type: Literal["mention_user"] = Field(default="mention_user", repr=False)
    villa_id: int
    user_id: int


class MentionAll(MessageSegment):
    """@全体成员消息段"""

    type: Literal["mention_all"] = Field(default="mention_all", repr=False)
    show_text: str = "全体成员"


class RoomLink(MessageSegment):
    """房间链接消息段"""

    type: Literal["room_link"] = Field(default="room_link", repr=False)
    villa_id: int
    room_id: int


class Link(MessageSegment):
    """链接消息段"""

    type: Literal["link"] = Field(default="link", repr=False)
    url: str
    show_text: str


class Image(MessageSegment):
    """图片消息段"""

    type: Literal["image"] = Field(default="image", repr=False)
    url: str
    width: Optional[int] = None
    height: Optional[int] = None
    file_size: Optional[int] = None


class Quote(MessageSegment):
    """引用消息段"""

    type: Literal["quote"] = Field(default="quote", repr=False)
    quoted_message_id: str
    quoted_message_send_time: int
    original_message_id: str
    original_message_send_time: int


class Post(MessageSegment):
    """帖子消息段"""

    type: Literal["post"] = Field(default="post", repr=False)
    post_id: str


class Message(BaseModel):
    __root__: List[MessageSegment] = Field(default_factory=list)

    def __init__(
        self,
        message: Union[
            str, MessageSegment, Iterable[MessageSegment], "Message", None
        ] = None,
    ):
        """消息类

        参数:
            message: 消息内容. 默认为 None.

        异常:
            TypeError: 不支持该类型的消息
        """
        if message is None:
            message = []
        elif isinstance(message, str):
            message = Text(content=message)
        if isinstance(message, MessageSegment):
            message = [message]
        elif isinstance(message, Message):
            message = message.__root__
        elif not isinstance(message, Iterable):
            raise TypeError(f"unsupported type: {type(message)}")
        super().__init__(__root__=message)

    def text(self, content: str) -> Self:
        """纯文本消息

        参数:
            content: 文本内容

        返回:
            Self: 消息对象

        用法:
            ```python
            message = Message().text("hello world")
            ```
        """
        self.__root__.append(Text(content=content))
        return self

    def mention_user(self, villa_id: int, user_id: int) -> Self:
        """提及(@at)消息

        参数:
            villa_id: 要提及的用户所在的大别野ID
            user_id: 要提及的用户ID

        返回:
            Self: 消息对象
        """
        self.__root__.append(MentionUser(villa_id=villa_id, user_id=user_id))
        return self

    def mention_all(self) -> Self:
        """提及(@at)全体成员消息

        返回:
            Self: 消息对象
        """
        self.__root__.append(MentionAll())
        return self

    def mention_robot(self, bot_id: str, bot_name: str) -> Self:
        """提及(@at)机器人消息

        参数:
            bot_id: 机器人ID
            bot_name: 机器人名称

        返回:
            Self: 消息对象
        """
        self.__root__.append(MentionRobot(bot_id=bot_id, bot_name=bot_name))
        return self

    def room_link(self, villa_id: int, room_id: int) -> Self:
        """房间链接消息

        参数:
            villa_id: 大别野ID
            room_id: 房间ID

        返回:
            Self: 消息对象
        """
        self.__root__.append(RoomLink(villa_id=villa_id, room_id=room_id))
        return self

    def link(self, url: str, show_text: Optional[str] = None) -> Self:
        """说明

        详细说明

        参数:
            url: 参数说明
            text: 参数说明. 默认为 None.

        返回:
            Self: 返回说明
        """
        self.__root__.append(Link(url=url, show_text=show_text or url))
        return self

    def image(
        self,
        url: str,
        width: Optional[int] = None,
        height: Optional[int] = None,
        file_size: Optional[int] = None,
    ) -> Self:
        """图片消息

        参数:
            url: 图片链接
            width: 图片宽度. 默认为 None.
            height: 图片高度. 默认为 None.
            file_size: 图片大小. 默认为 None.

        返回:
            Self: 消息对象
        """
        self.__root__.append(
            Image(url=url, width=width, height=height, file_size=file_size)
        )
        return self

    def quote(self, message_id: str, message_send_time: int) -> Self:
        """引用消息

        参数:
            message_id: 被引用的消息ID
            message_send_time: 被引用消息的发送时间

        返回:
            Self: 消息对象
        """
        self.__root__.append(
            Quote(
                quoted_message_id=message_id,
                quoted_message_send_time=message_send_time,
                original_message_id=message_id,
                original_message_send_time=message_send_time,
            )
        )
        return self

    def post(self, post_id: str) -> Self:
        """帖子转发消息

        参数:
            post_id: 帖子ID

        返回:
            Self: 消息对象
        """
        self.__root__.append(Post(post_id=post_id))
        return self

    def insert(self, index: int, segment: Union[str, MessageSegment]):
        """在指定位置插入消息段

        参数:
            index: 插入位置
            segment: 消息段

        返回:
            Self: 消息对象
        """
        if isinstance(segment, str):
            segment = Text(content=segment)
        self.__root__.insert(index, segment)

    def append(self, segment: Union[str, MessageSegment]):
        """在消息末尾添加消息段

        参数:
            segment: 消息段

        返回:
            Self: 消息对象
        """
        if isinstance(segment, str):
            segment = Text(content=segment)
        self.__root__.append(segment)

    def plain_text(self) -> str:
        """获取纯文本消息内容"""
        return "".join(
            [segment.content for segment in self.__root__ if isinstance(segment, Text)]
        )

    def __contains__(self, item: str) -> bool:
        """检查消息的纯文本内容是否包含指定字符串"""
        return item in self.plain_text()

    def __len__(self) -> int:
        return len(self.__root__)

    def __add__(self, other: Union[str, "MessageSegment", "Message"]) -> "Message":
        result = self.copy(deep=True)
        if isinstance(other, str):
            other = Text(content=other)
        if isinstance(other, MessageSegment):
            result.__root__.append(other)
        elif isinstance(other, Message):
            result.__root__.extend(other.__root__)
        else:
            raise TypeError(f"unsupported type: {type(other)}")
        return result

    def __radd__(self, other: Union[str, "MessageSegment", "Message"]) -> "Message":
        return self.__add__(other)

    def __iadd__(self, other: Union[str, "MessageSegment", "Message"]) -> Self:
        if isinstance(other, str):
            other = Text(content=other)
        if isinstance(other, MessageSegment):
            self.__root__.append(other)
        elif isinstance(other, Message):
            self.__root__.extend(other.__root__)
        else:
            raise TypeError(f"unsupported type: {type(other)}")
        return self

    def __iter__(self) -> Iterator[MessageSegment]:
        return iter(self.__root__)

    def __repr__(self) -> str:
        return f"Message({repr(self.__root__)})"

    def has_segment_type(self, segment_type: MessageType) -> bool:
        """判断消息是否包含指定类型的消息段

        参数:
            segment_type: 消息段类型

        返回:
            bool: 是否包含指定类型的消息段
        """
        return any(seg.type == segment_type for seg in self.__root__)

    def startswith(self, text: Union[str, Tuple[str, ...]]) -> bool:
        """判断消息的纯文本部分是否以指定字符串开头

        参数:
            text: 指定字符串

        返回:
            bool: 是否以指定字符串开头
        """
        return self.plain_text().startswith(text)

    def endswith(self, text: Union[str, Tuple[str, ...]]) -> bool:
        """判断消息的纯文本部分是否以指定字符串结尾

        参数:
            text: 指定字符串

        返回:
            bool: 是否以指定字符串结尾
        """
        return self.plain_text().endswith(text)

    def match(self, pattern: Union[str, re.Pattern]) -> Optional[re.Match]:
        """使用正则表达式匹配消息的纯文本部分

        参数:
            pattern: 正则表达式

        返回:
            Optional[re.Match]: 匹配结果
        """
        return re.match(pattern, self.plain_text())

    def search(self, pattern: Union[str, re.Pattern]) -> Optional[re.Match]:
        """使用正则表达式搜索消息的纯文本部分

        参数:
            pattern: 正则表达式

        返回:
            Optional[re.Match]: 匹配结果
        """
        return re.search(pattern, self.plain_text())

    @overload
    def __getitem__(self, __args: int) -> MessageSegment:
        """
        参数:
            __args: 索引

        返回:
            MessageSegment: 第 `__args` 个消息段
        """
        ...

    @overload
    def __getitem__(self, __args: slice) -> "Message":
        """
        参数:
            __args: 切片

        返回:
            Message: 消息切片 `__args`
        """
        ...

    @overload
    def __getitem__(self, __args: MessageType) -> "Message":
        """
        参数:
            __args: 消息类型

        返回:
            Message: 消息段类型为 `__args` 的消息
        """
        ...

    @overload
    def __getitem__(self, __args: Tuple[MessageType, slice]) -> "Message":
        """
        参数:
            __args: 消息段类型, 切片

        返回:
            Message: 消息段类型为 `__args` 的消息切片
        """
        ...

    @overload
    def __getitem__(self, __args: Tuple[Literal["text"], int]) -> Optional[Text]:
        """
        参数:
            __args: text消息段

        返回:
            Message: 消息段类型为text的第 `__args[1]` 个消息段
        """
        ...

    @overload
    def __getitem__(
        self, __args: Tuple[Literal["mention_user"], int]
    ) -> Optional[MentionUser]:
        """
        参数:
            __args: mention_user消息段

        返回:
            Message: 消息段类型为mention_user的第 `__args[1]` 个消息段
        """
        ...

    @overload
    def __getitem__(
        self, __args: Tuple[Literal["mention_all"], int]
    ) -> Optional[MentionAll]:
        """
        参数:
            __args: mention_all消息段

        返回:
            Message: 消息段类型为mention_all的第 `__args[1]` 个消息段
        """
        ...

    @overload
    def __getitem__(
        self, __args: Tuple[Literal["mention_robot"], int]
    ) -> Optional[MentionRobot]:
        """
        参数:
            __args: mention_robot消息段

        返回:
            Message: 消息段类型为mention_robot的第 `__args[1]` 个消息段
        """
        ...

    @overload
    def __getitem__(
        self, __args: Tuple[Literal["room_link"], int]
    ) -> Optional[RoomLink]:
        """
        参数:
            __args: room_link消息段

        返回:
            Message: 消息段类型为room_link的第 `__args[1]` 个消息段
        """
        ...

    @overload
    def __getitem__(self, __args: Tuple[Literal["link"], int]) -> Optional[Link]:
        """
        参数:
            __args: link消息段

        返回:
            Message: 消息段类型为link的第 `__args[1]` 个消息段
        """
        ...

    @overload
    def __getitem__(self, __args: Tuple[Literal["image"], int]) -> Optional[Image]:
        """
        参数:
            __args: image消息段

        返回:
            Message: 消息段类型为image的第 `__args[1]` 个消息段
        """
        ...

    @overload
    def __getitem__(self, __args: Tuple[Literal["quote"], int]) -> Optional[Quote]:
        """
        参数:
            __args: quote消息段

        返回:
            Message: 消息段类型为quote的第 `__args[1]` 个消息段
        """
        ...

    def __getitem__(
        self,
        args: Union[int, slice, MessageType, Tuple[MessageType, Union[int, slice]]],
    ) -> Union[MessageSegment, "Message", None]:
        arg1, arg2 = args if isinstance(args, tuple) else (args, None)
        if isinstance(arg1, int) and arg2 is None:
            return self.__root__.__getitem__(arg1)
        elif isinstance(arg1, slice) and arg2 is None:
            return Message(self.__root__.__getitem__(arg1))
        elif isinstance(arg1, str) and arg1 in (
            "text",
            "mention_user",
            "mention_all",
            "mention_robot",
            "room_link",
            "link",
            "image",
            "quote",
            "post",
        ):
            if arg2 is None:
                return Message([seg for seg in self.__root__ if seg.type == arg1])
            elif isinstance(arg2, int):
                try:
                    return [seg for seg in self.__root__ if seg.type == arg1][arg2]
                except IndexError:
                    return None
            elif isinstance(arg2, slice):
                return Message([seg for seg in self.__root__ if seg.type == arg1][arg2])
            else:
                raise ValueError("Incorrect arguments to slice")
        else:
            raise ValueError("Incorrect arguments to slice")
